#!/usr/bin/env python
import datetime
import html
import json
import uuid

import molstruct.names as n


def jsonldhtml(reader, limit=None):
    print('''<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Example Document</title>
    <script type="application/ld+json">''')
    jsonld(reader, limit=None)
    print('''    </script>
  </head>
</html>''')


def jsonld(reader, limit=None):
    i = 0

    out_str = '{\n'
    out_str += '  "@graph" : [\n'
    out_str += '''    {
      "@id": "https://github.com/lszeremeta/molstruct",
      "@type": "http://schema.org/Organization",
      "http://schema.org/name": "Molstruct"
    },
    {
      "@id": "#",
      "@type": "http://schema.org/Dataset",
      "http://schema.org/about": {
        "@id": "https://github.com/lszeremeta/molstruct"
      },
      "http://schema.org/description": "This is a dataset of molecules generated by Molstruct.",
      "http://schema.org/keywords": [
        "molecules",
        "cheminformatics",
        "chemical compounds"
      ],
      "http://schema.org/license": {
        "@id": "https://creativecommons.org/licenses/by/3.0/"
      },
      "http://schema.org/name": "Molecules",
      "http://schema.org/creator": {
        "@id": "https://github.com/lszeremeta/molstruct"
      },
      "http://schema.org/temporal": "''' + str(datetime.datetime.today().year) + '''",
      "http://schema.org/url": "https://github.com/lszeremeta/molstruct"
    },'''
    for row in reader:
        out_str += '\n    {\n'

        if n.SUBJECT_BASE:
            out_str += '      "@id" : ' + json.dumps(n.SUBJECT_BASE + str(i))
        else:
            out_str += '      "@id" : ' + json.dumps('urn:uuid:' + str(uuid.uuid4()))

        out_str += ',\n      "@type" : "http://schema.org/MolecularEntity",\n'

        for key, value in n.COLUMNS.items():
            if n.VALUE_DELIMITER in str(row.get(value)):
                out_str += '      "' + key + '" : ' + json.dumps(row.get(value).split(n.VALUE_DELIMITER)) + ',\n'
            elif row.get(value):
                out_str += '      "' + key + '" : ' + json.dumps(row.get(value)) + ',\n'

        out_str = out_str[:-2] + '\n'
        out_str += '    },'

        if i + 1 == limit:
            break

        i = i + 1
    out_str = out_str[:-1]

    print(out_str + '''\n  ],
  "@context" : {
    "identifier" : {
      "@id" : "http://schema.org/identifier"
    },
    "name" : {
      "@id" : "http://schema.org/name"
    },
    "inChIKey" : {
      "@id" : "http://schema.org/inChIKey"
    },
    "inChI" : {
      "@id" : "http://schema.org/inChI"
    },
    "smiles" : {
      "@id" : "http://schema.org/smiles"
    },
    "url" : {
      "@id" : "http://schema.org/url"
    },
    "iupacName" : {
      "@id" : "http://schema.org/iupacName"
    },
    "molecularFormula" : {
      "@id" : "http://schema.org/molecularFormula"
    },
    "molecularWeight" : {
      "@id" : "http://schema.org/molecularWeight"
    },
    "monoisotopicMolecularWeight" : {
      "@id" : "http://schema.org/monoisotopicMolecularWeight"
    },
    "description" : {
      "@id" : "http://schema.org/description"
    },
    "disambiguatingDescription" : {
      "@id" : "http://schema.org/disambiguatingDescription"
    },
    "image" : {
      "@id" : "http://schema.org/image"
    },
    "alternateName" : {
      "@id" : "http://schema.org/alternateName"
    },
    "sameAs" : {
      "@id" : "http://schema.org/sameAs"
    },
    "schema" : "http://schema.org/"
  }
}''')


def rdfa(reader, limit=None):
    i = 0
    print('''<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Example Document</title>
  </head>
  <body vocab="http://schema.org/">
    <div typeof="schema:Dataset">
      <div rel="schema:creator">
        <div typeof="schema:Organization" about="https://github.com/lszeremeta/molstruct">
          <div property="schema:name" content="Molstruct"></div>
        </div>
      </div>
      <div property="schema:keywords" content="cheminformatics"></div>
      <div property="schema:keywords" content="molecules"></div>
      <div property="schema:keywords" content="chemical compounds"></div>
      <div property="schema:temporal" content="''' + str(datetime.datetime.today().year) + '''"></div>
      <div property="schema:name" content="Molecules"></div>
      <div rel="schema:license" resource="https://creativecommons.org/licenses/by/3.0/"></div>
      <div property="schema:description" content="This is a dataset of molecules generated by Molstruct."></div>
      <div rel="schema:about" resource="https://github.com/lszeremeta/molstruct"></div>
      <div property="schema:url" content="https://github.com/lszeremeta/molstruct"></div>
    </div>''')
    for row in reader:
        if n.SUBJECT_BASE:
            print('    <div typeof="schema:MolecularEntity" about="' + html.escape(n.SUBJECT_BASE + str(i)), end='')

            if '#' in n.SUBJECT_BASE:
                print('" id="' + html.escape(n.SUBJECT_BASE.rpartition('#')[-1] + str(i)), end='')

        else:
            print('    <div typeof="schema:MolecularEntity" about="urn:uuid:' + str(uuid.uuid4()), end='')

        print('">')

        for key, value in n.COLUMNS.items():
            if row.get(value):
                values = row.get(value).split(n.VALUE_DELIMITER)
                for v in values:
                    if key == 'url' or key == 'sameAs':
                        print('      <a href="' + html.escape(v) + '" rel="schema:' + key + '">' + html.escape(
                            v, quote=False) + '</a>')
                    elif key == 'image':
                        print('      <img src="' + html.escape(v) + '" alt="Image of the Molecule' + str(
                            i) + '" rel="schema:' + key + '">')
                    else:
                        print('      <div property="schema:' + key + '">' + html.escape(v, quote=False) + '</div>')

        print('    </div>')

        if i + 1 == limit:
            break

        i = i + 1
    print('  </body>')
    print('</html>')


def microdata(reader, limit=None):
    i = 0
    print('''<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Example Document</title>
  </head>
  <body>
    <div itemscope itemtype="http://schema.org/Dataset">
      <div itemprop="name" content="Molecules"></div>
      <div itemprop="keywords" content="cheminformatics"></div>
      <div itemprop="keywords" content="molecules"></div>
      <div itemprop="keywords" content="chemical compounds"></div>
      <div itemprop="temporal" content="''' + str(datetime.datetime.today().year) + '''"></div>
      <div itemprop="url" content="https://github.com/lszeremeta/molstruct"></div>
      <div itemprop="description" content="This is a dataset of molecules generated by Molstruct."></div>
      <div itemprop="creator" itemscope itemtype="http://schema.org/Organization">
        <div itemprop="name" content="Molstruct"></div>
      </div>
      <div itemprop="license" content="https://creativecommons.org/licenses/by/3.0/"></div>
    </div>''')
    for row in reader:
        if n.SUBJECT_BASE:
            print('    <div itemscope itemtype="http://schema.org/MolecularEntity" itemid="' + html.escape(
                n.SUBJECT_BASE + str(i)), end='')

            if '#' in n.SUBJECT_BASE:
                print('" id="' + html.escape(n.SUBJECT_BASE.rpartition('#')[-1] + str(i)), end='')
        else:
            print(
                '    <div itemscope itemtype="http://schema.org/MolecularEntity" itemid="urn:uuid:' + str(uuid.uuid4()),
                end='')

        print('">')

        for key, value in n.COLUMNS.items():
            if row.get(value):
                values = row.get(value).split(n.VALUE_DELIMITER)
                for v in values:
                    if key == 'url' or key == 'sameAs':
                        print(
                            '      <a href="' + html.escape(v) + '" itemprop="' + key + '">' + html.escape(
                                v, quote=False) + '</a>')
                    elif key == 'image':
                        print('      <img src="' + html.escape(v) + '" alt="Image of the Molecule' + str(
                            i) + '" itemprop="' + key + '">')
                    else:
                        print('      <div itemprop="' + key + '">' + html.escape(v, quote=False) + '</div>')

        print('    </div>')

        if i + 1 == limit:
            break

        i = i + 1
    print('  </body>')
    print('</html>')
